@@ -3,12 +3,12 @@ use Data::Dumper;
 use strict;
 use warnings;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 =head1 NAME
 
 Crypt::LE::Challenge::Simple - A boilerplate for extending Crypt::LE and Crypt::LE client
-application (le.pl) with challenge/varification handlers.
+application (le.pl) with challenge/verification handlers.
 
 =head1 SYNOPSIS
 
@@ -41,7 +41,7 @@ sub handle_challenge_http {
     if ($params) {
         print "Parameters passed: " . Dumper($params);
     }
-    print "Domain $challenge->{domain} expects '$challenge->{token}.$challenge->{fingerprint}' text in '$challenge->{token}' file\n";
+    print "Challenge for '$challenge->{domain}' requires a file '$challenge->{token}' in '/.well-known/acme-challenge/' with the text '$challenge->{token}.$challenge->{fingerprint}'\n";
     print "Press <Enter>";
     <STDIN>;
     return 1;
@@ -66,7 +66,6 @@ sub handle_verification_http {
     }
     print "Domain $verification->{domain} " . ($verification->{valid} ? "has been verified successfully." : "has NOT been verified.") . "\n";
     print "You can now delete '$verification->{token}' file\n";
-    <STDIN>;
     return 1;
 }
 
