@@ -1,8 +1,9 @@
 package Crypt::LE::Complete::Simple;
+use Data::Dumper;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 NAME
 
@@ -27,6 +28,9 @@ sub new { bless {}, shift }
 sub complete {
     my $self = shift;
     my ($data, $params) = @_;
+    if ($params) {
+        print "Parameters passed: " . Dumper($params);
+    }
     print "Domain Certificate '$data->{certificate_file}':\n$data->{certificate}\n";
     print "Issuer's Certificate:\n$data->{issuer}\n";
     print "Key file: '$data->{key_file}'.\n";
