@@ -4,21 +4,29 @@ use warnings;
 use Getopt::Long;
 use IO::File;
 use JSON::MaybeXS;
+use HTTP::Tiny;
+use Net::SSLeay;
+use Time::Piece;
+use Time::Seconds;
 use Crypt::LE ':errors';
 
 sub info { print shift, "\n" }
 sub bail { &info; exit 255 }
 
-my $VERSION = '0.10';
+my $VERSION = '0.11';
 
-my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, 
-    $handle_with, $handle_as, $handle_params, $complete_with, $complete_params, $revoke, $verified, $debug, $help);
+use constant PEER_CRT  => 4;
+use constant CRT_DEPTH => 5;
+
+my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, $renew,
+    $expires_in, $handle_with, $handle_as, $handle_params, $complete_with, $complete_params, $revoke, $verified, 
+    $unlink, $debug, $help);
 
 GetOptions ("key=s" => \$account_key, "csr=s" => \$csr, "csr-key=s" => \$csr_key, "domains=s" => \$domains, "path=s" => \$challenge_path,
-            "crt=s" => \$crt, "generate-missing" => \$generate_missing, "generate-only" => \$generate_only, "handle-with=s" => \$handle_with,
-	    "handle-as=s" => \$handle_as, "handle-params=s" => \$handle_params, "complete-with=s" => \$complete_with, 
-            "complete-params=s" => \$complete_params, "revoke" => \$revoke, "live" => \$live, "debug" => \$debug, "verified" => \$verified, 
-            "help" => \$help);
+            "crt=s" => \$crt, "generate-missing" => \$generate_missing, "generate-only" => \$generate_only, "renew=i" => \$renew,
+            "handle-with=s" => \$handle_with, "handle-as=s" => \$handle_as, "handle-params=s" => \$handle_params, "complete-with=s" => \$complete_with, 
+            "complete-params=s" => \$complete_params, "revoke" => \$revoke, "live" => \$live, "unlink" => \$unlink, "verified" => \$verified, 
+            "debug" => \$debug, "help" => \$help) or $help = 1;
 
 my $usage = <<EOF;
 
@@ -30,7 +38,12 @@ a) To register (if needed) and issue a certificate:
 
    le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing
 
-b) To use external modules to handle challenges and process completion while getting a certificate:
+b) To have challenge files automatically placed into your web directory before the verification and then removed after the verification:
+
+   le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing \\
+         --path /some/path/.well-known/acme-challenge --unlink
+
+c) To use external modules to handle challenges and process completion while getting a certificate:
 
    le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing \\
          --handle-with Crypt::LE::Challenge::Simple --complete-with Crypt::LE::Complete::Simple
@@ -38,22 +51,40 @@ b) To use external modules to handle challenges and process completion while get
    - See provided Crypt::LE::Challenge::Simple for an example of a challenge-handling module.
    - See provided Crypt::LE::Complete::Simple for an example of a completion-handling module.
 
-c) To pass parameters to external modules as JSON either directly or by specifying a file name:
+d) To pass parameters to external modules as JSON either directly or by specifying a file name:
 
    le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing \\
          --handle-with Crypt::LE::Challenge::Simple --complete-with Crypt::LE::Complete::Simple \\
          --handle-params '{"key1": 1, "key2": 2, "key3": "something"}' --complete-params complete.json
 
-d) To just generate the keys and CSR:
+e) To just generate the keys and CSR:
 
    le.pl  --key account.key --csr domain.csr --csr-key domain.key --domains "www.domain.ext,domain.ext" --generate-missing --generate-only
 
-e) To revoke a certificate:
+f) To revoke a certificate:
 
    le.pl --key account.key --crt domain.crt --revoke
 
+===============
+RENEWAL PROCESS
+===============
 
-[!] NOTE: by default a staging server is used, which does not provide trusted certificates. This is to aviod hitting a 
+To RENEW your existing certificate: use the same command line as you used for issuing the certificate, with one additional parameter
+   
+   --renew XX, where XX is the number of days left until certificate expiration.
+
+If le.pl detects that it is XX or fewer days left until certificate expiration, then (and only then) the renewal process will be run,
+so the script can get safely put into crontab to run on a daily basis if needed. The amount of days left is checked by either of two
+methods:
+
+   1) If the certificate (which name is used with --crt parameter) is available locally, then it will be loaded and checked.
+
+   2) If the certificate is not available locally (for example if you moved it to another server), then an attempt to
+      connect to the domains listed in --domains or CSR will be made until the first successful response is received. The
+      peer certificate will be then checked for expiration.
+
+
+[!] NOTE: by default a staging server is used, which does not provide trusted certificates. This is to avoid hitting a 
           rate limits on Let's Encrypt live server. To generate an actual certificate, always add --live option.
 
 Available parameters:
@@ -70,6 +101,8 @@ Available parameters:
  complete-params <{json}|file>    - JSON (or name of the file containing it) with parameters to be passed to the completion-handling module (optional).
  generate-missing                 - Generate missing files (key, csr and csr-key).
  generate-only                    - Generate a new key and/or CSR if they are missing and then exit.
+ unlink                           - Remove challenge files which were automatically created if --path option was used.
+ renew <XX>                       - Renew the certificate if XX or fewer days are left until its expiration.
  crt <file>                       - Name for the domain certificate file.
  verified                         - Skip challenge verification steps (request/accept/verify). Use only if domains have been already verified recently.
  revoke                           - Revoke a certificate.
@@ -108,6 +141,8 @@ if ($revoke) {
 
     if ($challenge_path) {
         bail "Path to save challenge files into should be a writable directory" unless (-d $challenge_path and -w _);
+    } elsif ($unlink) {
+        bail "Unlink option will have no effect without --path.";
     }
 
     $handle_as = $handle_as ? lc($handle_as) : 'http';
@@ -174,6 +209,33 @@ if (-r $csr) {
 
 exit if $generate_only;
 
+if ($renew) {
+    my $rv = 1;
+    if ($crt and -r $crt) {
+        info "Checking certificate for expiration (local file).";
+        $rv = verify_crt_file($crt);
+        info "Problem checking existing certificate file: $rv" if $rv;
+    }
+    if ($rv) {
+        info "Checking certificate for expiration (website connection).";
+        my $probe = HTTP::Tiny->new( agent => "Crypt::LE v$VERSION renewal agent", verify_SSL => 1, timeout => 10, SSL_options => { SSL_verify_callback => \&verify_crt } );
+        foreach my $domain (@{$le->domains}) {
+            info "Checking $domain";
+            $probe->head("https://$domain/");
+            if (defined $expires_in) {
+                $rv = 0;
+                last;
+            }
+        }
+    }
+    bail "Could not get the certificate expiration value, cannot renew." if $rv;
+    if ($expires_in > $renew) {
+        info "Too early for renewal, certificate expires in $expires_in days.";
+        exit 0;
+    }
+    info "Expiration threshold set at $renew days, the certificate " . ($expires_in < 0 ? "has already expired" : "expires in $expires_in days") . " - will be renewing.";
+}
+
 bail $le->error_details if $le->register;
 $le->accept_tos(); # No need to check explicitly for tos_changed() - Crypt::LE will handle that for you.
 unless ($verified) {
@@ -248,6 +310,30 @@ sub _write {
     return 0;
 }
 
+sub verify_crt {
+    my @crt = @_;
+    unless ($crt[CRT_DEPTH]) {
+        undef $expires_in;
+        my ($t, $s);
+        eval {
+            $t = Net::SSLeay::X509_get_notAfter($crt[PEER_CRT]);
+            $t = Time::Piece->strptime(Net::SSLeay::P_ASN1_TIME_get_isotime($t), "%Y-%m-%dT%H:%M:%SZ");
+        };
+        return $@ if $@;
+        $s = $t - localtime;
+        $expires_in = int($s->days);
+        return 0;
+    }
+}
+
+sub verify_crt_file {
+    my $file = shift;
+    my $bio = Net::SSLeay::BIO_new_file($file, 'r') or return $!;
+    my $cert = Net::SSLeay::PEM_read_bio_X509($bio);
+    Net::SSLeay::BIO_free($bio);
+    return $cert ? verify_crt(0, 0, 0, 0, $cert, 0) : "Could not parse the certificate";
+}
+
 sub process_challenge {
     my $challenge = shift;
     my $text = "$challenge->{token}.$challenge->{fingerprint}";
@@ -274,7 +360,14 @@ sub process_verification {
     my $results = shift;
     info  "Domain verification results for '$results->{domain}': " . ($results->{valid} ? "success." : "error.");
     my $file = $challenge_path ? "$challenge_path/$results->{token}" : $results->{token};
-    # NB: We don't delete challenge files automatically, this option might be added later.
-    info "You can now delete the '$file' file.";
+    if ($unlink) {
+        if (unlink $file) {
+            info "Challenge file '$file' has been deleted.";
+        } else {
+            info "Could not delete the challenge file '$file', you may need to do it manually.";
+        }
+    } else {
+        info "You can now delete the '$file' file.";
+    }
     1;
 }
